#*********************************************************************************************
#*********************************************************************************************
#**
#**  File     :  /maps/Solo ACU/Solo ACU_script.lua
#**
#**  Author(s):  Manimal (scripter) + Saya (help) + ertwyu, BennyMoore, KrimsonStorm (game concept)
#**
#**  Version  :  4
#**
#**  Date     :  26/08/2007
#**
#**  Summary  :  Solo ACU Map Script
#**
#**********************************************************************************************
#**********************************************************************************************


--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Definitions                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


local ScenarioUtils = import('/lua/sim/ScenarioUtilities.lua')
local ScenarioFramework = import('/lua/ScenarioFramework.lua')
local Utilities = import('/lua/Utilities.lua')
local Cinematics = import( '/lua/cinematics.lua' )





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Global Definitions                                                   ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


ScenarioInfo.Attacker = 1
ScenarioInfo.Defender = 2

theAttacker = ScenarioInfo.Attacker
theDefender = ScenarioInfo.Defender



--[[                                                                             ]]--
--[[   ubcsTable                                                                 ]]--
--[[   { name, enemi, unitID,                                                    ]]--
--[[          EnhancementsList, EnhRestrictionsList, BuildRestrictionsList }     ]]--
--[[                                                                             ]]--
--[[   Do NOT erase any line in this ubcsTable !                                 ]]--
--[[   Add a comment sign instead for an evenutal future use                     ]]--
--[[                                                                             ]]--

ubcsTable = {

    { name                  = 'ATTACKER',
      army                  = 'ATTACKER',
      enemi                 = 'DEFENDER',
      unitId                = 'url0001', 
      isReady               = false,
      EnhancementsList      = { 'AdvancedEngineering',
                                'CoolingUpgrade',
                                'MicrowaveLaserGenerator',
                                'ResourceAllocation',
                                'T3Engineering'
                              },
      EnhRestrictionsList   = { 'NaniteTorpedoTube',
                                'StealthGenerator'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.UEF,
                                categories.NAVAL,
                                categories.EXPERIMENTAL,
                                categories.ura0102,  # Interceptor
                                categories.ura0103,  # Attack Bomber
                                categories.ura0203,  # Gunship
                                categories.ura0204,  # Torpedo Bomber
                                categories.dra0202,  # Bomber Corsair
                                categories.ura0303,  # Air Superiority Fighter
                                categories.ura0304,  # Strategic Bomber
                                categories.urb0304,  # Quantum Gateway
                                categories.urb1102,  # Hydrocarbon Power Plant
                                categories.urb2108,  # Tactical Missile Launcher
                                categories.urb2109,  # Tech 1 Torpedo Launcher
                                categories.urb2205,  # Tech 2 Torpedo Launcher
                                categories.urb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.urb2302,  # Long Range Artillery
                                categories.urb2303,  # Light Artillery
                                categories.urb2305,  # Strategic Missile Launcher
                                categories.urb3102,  # Tech 1 Sonar System
                                categories.urb3104,  # Omni Sensor Array
                                categories.urb3202,  # Tech 2 Sonar System
                                categories.urb4203,  # Stealth Field Generator
                                categories.urb4302,  # Strategic Missile Defense
                                categories.urb5101,  # Wall Section
                                categories.urb5103,  # Quantum Gate Beacon
                                categories.url0103,  # Mobile Light Artillery
                                categories.url0106,  # Light Assault Bot
                                categories.url0107,  # Heavy Assault Bot
                                categories.url0111,  # Mobile Missile Launcher
                                categories.url0202,  # Heavy Tank
                                categories.url0203,  # Amphibious Tank
                                categories.url0301,  # Support Command Unit
                                categories.url0303,  # Siege Assault Bot
                                categories.url0304,  # Mobile Heavy Artillery
                                categories.url0306,  # Mobile Stealth Field System
                                categories.url5103,  # Quantum Gate Beacon
                                categories.urs0305   # Sonar Platform
                              }
    },

    { name                  = 'DEFENDER', 
      army                  = 'DEFENDER',
      enemi                 = 'ATTACKER',
      unitId                = 'uel0001', 
      isReady               = false,
      EnhancementsList      = { 'AdvancedEngineering'},
      EnhRestrictionsList   = { 'DamageStablization',
                                'HeavyAntiMatterCannon',
                                'T3Engineering',
                                'TacticalNukeMissile'
                              },
      BuildRestrictionsList = { categories.AEON,
                                categories.CYBRAN,
                                categories.NAVAL,
                                categories.TECH3,
                                categories.EXPERIMENTAL,
                                categories.ueb1102,  # Hydrocarbon Power Plant
                                categories.ueb1201,  # Tech 2 Power Plant
                                categories.ueb2109,  # Tech 1 Torpedo Launcher
                                categories.ueb2205,  # Tech 2 Torpedo Launcher
                                categories.ueb2301,  # Tech 2 Point Defense (Heavy Gun Tower)
                                categories.ueb2302,  # Long Range Artillery
                                categories.ueb2303,  # Light Artillery
                                categories.ueb3102,  # Tech 1 Sonar System
                                categories.ueb3202,  # Tech 2 Sonar System
                                categories.ueb5101,  # Wall Section
                                categories.uel0307   # Mobile Shield Generator
                              }
    }

}

# end of ubcsTable


--[[  Table of Units to be spawned  ]]--
--[[  same format as ubcsTable      ]]--
--[[  so these are concatenable     ]]--

unitsTable = {

    { name                  = 'Air_Transport_1',
      army                  = ubcsTable[theAttacker].army,
      enemi                 = ubcsTable[theDefender].army,
      unitId                = 'ura0104',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { }
    },

    { name                  = 'Air_Spies_1', 
      army                  = ubcsTable[theAttacker].army,
      enemi                 = ubcsTable[theDefender].army,
      unitId                = 'ura0302',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { }
    },

    { name                  = 'Tactical_Missiles_1', 
      army                  = ubcsTable[theDefender].army,
      enemi                 = ubcsTable[theAttacker].army,
      unitId                = 'ueb2108',
      isReady               = false,
      EnhancementsList      = { },
      EnhRestrictionsList   = { },
      BuildRestrictionsList = { }
    }

}


--[[  Miscellaneous Variables and Flags  ]]--

ACUisReady            = false

attackUBC             = false

defensUBC             = false

A_Air_Transport_1     = false

A_Air_Spies_1         = { 
                          Air_Spy_01 = false,
                          Air_Spy_02 = false,
                          Air_Spy_03 = false,
                          Air_Spy_04 = false,
                          Air_Spy_05 = false,
                          Air_Spy_06 = false,
                          Air_Spy_07 = false,
                          Air_Spy_08 = false,
                          Air_Spy_09 = false,
                          Air_Spy_10 = false
                        }

D_Tactical_Missiles_1 = false





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Local Tuning Variables                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--


# The players's unit cap
local AttackerUnitCap = 500
local DefenderUnitCap = 500





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   Misc Local Definitions                                                    ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--







--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[   STARTER FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--





--[[                                                                             ]]--
--[[   OnPopulate()                                                              ]]--
--[[                                                                             ]]--
--[[   alliances and unit spawning go in there so you dont get conflicts and     ]]--
--[[   weird behaviours with initializearmies                                    ]]--
--[[                                                                             ]]--

function OnPopulate()


    --[[  Custom Initialization of Armies  ]]--

    InitializeArmies()


    --[[  GENERAL SETUP                                                          ]]--
    --[[                                                                         ]]--
    --[[  Scores, UnitCap, Alliances, etc                                        ]]--


    --[[  Fix Scenario's Options  ]]--

    ScenarioInfo.Options.Victory          = 'assassination'
    ScenarioInfo.Options.TeamSpawn        = 'fixed'
    ScenarioInfo.Options.TeamLock         = 'locked'
    ScenarioInfo.Options.FogOfWar         = 'explored' 
    ScenarioInfo.Options.CivilianAlliance = 'neutral' 
    ScenarioInfo.Options.CheatsEnabled    = 'false' 
    ScenarioInfo.Options.NoRushOption     = 'Off' 
    ScenarioInfo.Options.PrebuiltUnits    = 'Off' 
    ScenarioInfo.Options.UnitCap          = '500' 

    #SetUnitCap(500)

    --[[  Set the playable AREA  ]]--

    #ScenarioFramework.SetPlayableArea( Rect(0, 0, 512, 512) )

    --[[  Fix players's Unit Cap, NO EXTRA UNITS ALLOWED  ]]--

    SetIgnoreArmyUnitCap( theAttacker, false )
    SetIgnoreArmyUnitCap( theDefender, false )
    SetArmyUnitCap( theAttacker, AttackerUnitCap )
    SetArmyUnitCap( theDefender, DefenderUnitCap )

    --[[  Set ALL Players's ALLIANCES  ]]--

    SetAlliance( ubcsTable[theAttacker].name, ubcsTable[theDefender].name, 'Enemy' )


    --[[  Transport UBC jusqu'� la Drop Zone  ]]--

    if attackUBC != nil then
        TransportUBCversDropZone( attackUBC )
    else
        LOG("MANIMAL:  TransportUBCversDropZone of attackUBC is not possible for player ".. iArmy .. " " .. strArmy .. " .")
    end


    --[[  Envoyer Patrouille Air Spies autour de la Drop Zone  ]]--

    AirSpiesPatrouilleDropZone()


    --[[  Armer Lance Missiles Tactique  ]]--

    ArmerLanceMissilesTactique()


    --[[  LOCK the CONTROLS until ALL Units are READY, and ACU in Drop Zone  ]]--
    --[[  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  ]]--

    # TEST SANS local
    WaitForACU = ForkThread( function (self) 

        if not ACUisReady then
           LockInput()
           while not ACUisReady do
               WaitSeconds(1)
               if ACUisReady then
                   break
               end
           end
        end  

        if ACUisReady then
            LOG("MANIMAL:  ACU is Ready => UNLOCKING INPUT !!! .")
            UnlockInput()
        else
            LOG("MANIMAL:  DEBUG TEST IF ACUisReady => FALSE (Locked!) .")
        end   

        KillThread(self)

    end)

    --[[  ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  ]]--

  
    --[[  Fixes customized name because the TransportLoad resets it for some unknown reason  ]]--
    if ACUisReady then
        attackUBC:SetCustomName( ArmyBrains[theAttacker].Nickname .. " - ATTACKER" )
        defensUBC:SetCustomName( ArmyBrains[theDefender].Nickname .. " - DEFENDER" )
    end


    --[[  Camera Tracking : Transport avec UBC � bord        ]]--
    --[[  CameraTrackEntity( unit, seconds, unlock )         ]]--

    LOG("MANIMAL:  Camera Tracking Entity A_Air_Transport_1 with ACU Attached to it .")
 
    if not A_Air_Transport_1:IsDead() then 

        ForkThread( function (self)
            local unlock = true

            CameraTrackEntity( attackUBC, 37, unlock )
            #CameraTrackEntity( A_Air_Transport_1, 37, unlock )

            KillThread(self)

        end)

    end

end

# END of onPopulate






--[[                                                                             ]]--
--[[   OnStart(self)                                                             ]]--
--[[                                                                             ]]--
--[[   Game mechanisms are there                                                 ]]--
--[[                                                                             ]]--

function OnStart(self)

    --[[  Show Enemy life bars  ]]--
    Utilities.UserConRequest("ui_ForceLifbarsOnEnemy")

    --[[  LET's ROCK !  ]]--

end






--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--
--[[                                                                             ]]--
--[[  OTHER FUNCTIONS                                                            ]]--
--[[                                                                             ]]--
--[[ *************************************************************************** ]]--
--[[ *************************************************************************** ]]--




--[[                                                                             ]]--
--[[   InitializeArmies()                                                        ]]--
--[[                                                                             ]]--
--[[   Spawn CUSTOM COMMANDERS and their ARMIES                                  ]]--
--[[                                                                             ]]--

function InitializeArmies()

    local tblGroups = {}
    local tblArmy = ListArmies()

    for iArmy, strArmy in pairs(tblArmy) do

        local tblData = Scenario.Armies[strArmy]

        if tblData then
		
            SetArmyEconomy( strArmy, tblData.Economy.mass, tblData.Economy.energy)

            --[[  Spawn Upgraded ACUs  ]]--

            local cdrUnit = false
			
            cdrUnit = SpawnUpgradedUBC ( strArmy, iArmy ) 

            if cdrUnit != nil then
                LOG("MANIMAL:  SPAWN de Enhanced UBC pour joueur ".. iArmy .. " " .. strArmy .. " COMPLET� !")
                if strArmy == ubcsTable[theAttacker].name then
                    attackUBC = cdrUnit
                elseif strArmy == ubcsTable[theDefender].name then
                    defensUBC = cdrUnit
                end
            end

            --[[  PLAYERS'S RESTRICTIONS                                      ]]--
            --[[  Take away units that the players shouldn't have access to   ]]--

            if ubcsTable[iArmy].BuildRestrictionsList then 
                for i,catRestrict in ubcsTable[iArmy].BuildRestrictionsList do
                    LOG("MANIMAL:  Add Build Restriction at index=" .. i .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    ScenarioFramework.AddRestriction( strArmy, catRestrict )
                end
            end

            --[[  Pre-Built UNITS/BUILDINGS  ]]--

            local preBuiltBase = false

            preBuiltBase =  ScenarioUtils.CreateArmySubGroup(strArmy, 'INITIAL')

            if preBuiltBase != nil then
                LOG("MANIMAL:  SPAWN de PREBUILT BASE pour joueur ".. iArmy .. " " .. strArmy .. " .")
            end

            --[[  Spawn Wreckage etc...  ]]--
            --[[
            local wreckageGroup = FindUnitGroup('WRECKAGE', Scenario.Armies[strArmy].Units)

            if wreckageGroup then

                local platoonList, tblResult, treeResult = CreatePlatoons(strArmy, wreckageGroup )

                for num,unit in tblResult do
                    unit:CreateWreckageProp(0)
                    unit:Destroy()
                end

            end   #end IF wreckageGroup
            --]]


        end   #end IF tblData

    end   #end FOR 


    return tblGroups

end




--[[                                                                             ]]--
--[[   SpawnUpgradedUBC ( strArmy, iArmy )                                       ]]--
--[[                                                                             ]]--
--[[   Spawn an Upgraded ACU                                                     ]]--
--[[                                                                             ]]--

function SpawnUpgradedUBC ( strArmy, iArmy )

    local ubcId = ''       # ACU Identifier
    local cdrUnit = false

    ubcId = ubcsTable[iArmy].unitId

    if (ubcId == 'ual0001') or (ubcId == 'uel0001') or (ubcId == 'url0001') then  # ACU Identifier is valid

        # spawn uef commander
        cdrUnit = CreateUnitHPR( ubcId, strArmy, mPos(strArmy)[1], mPos(strArmy)[2],mPos(strArmy)[3], 0,0,0)

        # successful spawn
        if cdrUnit != nil then

            LOG("MANIMAL:  SPAWN de UBC pour joueur ".. iArmy .. " " .. strArmy .. " .")

            --[[  DO ENHANCEMENTS  ]]--

            if ubcsTable[iArmy].EnhancementsList then 
                for i,enhancement in ubcsTable[iArmy].EnhancementsList do
                    LOG("MANIMAL:  ACU Enhancement at index=" .. i .. ", enhancement=" .. enhancement .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    cdrUnit:CreateEnhancement(enhancement)
                end
            end

            # customize name as explicitly as possible
            # I've repeated the customized name feature in another part of
            # the script because it's reset for some unknown reason !
# TESTER table.getn(ArmyBrains[iArmy].Nickname)
            cdrUnit:SetCustomName( ArmyBrains[iArmy].Nickname .. " - " .. strArmy )

            # annihilation type victory condition
            cdrUnit.OldOnKilled = cdrUnit.OnKilled

            # action when killed
            cdrUnit.OnKilled = function(self, instigator, type, overkillRatio)
                GetArmyBrain(strArmy):OnDefeat()
                GetArmyBrain(strEnemi(iArmy)):OnVictory()
                cdrUnit.OldOnKilled(self, instigator, type, overkillRatio)	
            end
				
            --[[  DO RESTRICTIONS  ]]--

            if ubcsTable[iArmy].EnhRestrictionsList then 
                for i,restriction in ubcsTable[iArmy].EnhRestrictionsList do
                    LOG("MANIMAL:  ACU RestrictEnhancement at index=" .. i .. ", restriction=" .. restriction .. " for player ".. iArmy .. " " .. strArmy .. " .")
                    ScenarioFramework.RestrictEnhancements(restriction)
                end
            end

        else
            LOG("FAILED: Unit Creation (ACU) for player ".. iArmy .. " " .. strArmy .. " .")
        end   #end IF cdrUnit

        # play warp in effect is DISABLED in order to make a faster TransportLoad
        #cdrUnit:PlayCommanderWarpInEffect()

   end   #end IF ubcId

   return cdrUnit

end




--[[                                                                             ]]--
--[[   TransportUBCversDropZone( cdrUnit )                                       ]]--
--[[                                                                             ]]--
--[[   Load and Transport the Attacker's ACU to the Drop Zone                    ]]--
--[[                                                                             ]]--

function TransportUBCversDropZone(cdrUnit)

    --[[  Spawn Air Transport for Attacker ACU  ]]--

    local spawnPosLoc    = mPos( 'DZ_P0' )
    local origMarkerPos  = ScenarioUtils.MarkerToPosition( 'DZ_P0' )
    local destMarkerPos  = ScenarioUtils.MarkerToPosition( 'DZ_P1' )
			
    A_Air_Transport_1 = CreateUnitHPR( unitsTable[1].unitId, unitsTable[1].army, spawnPosLoc[1], spawnPosLoc[2], spawnPosLoc[3], 0, 0, 0 )

    --[[  Successful ACU spawn so the transport can be loaded up !  ]]--

    if A_Air_Transport_1 != nil then

        LOG("MANIMAL:  SPAWN de A_Air_Transport_1 .")

        --[[  LOADING UP ATTACKER IN AIR TRANSPORT  ]]--

        if not cdrUnit:IsDead() then

            --[[  AttachUnitsToTransports(units, transports)                       ]]--
            --[[  units is a set of units, transports is a set of transports       ]]--
            --[[  Automatically attaches <units> to attach points on <transports>  ]]--

            ScenarioFramework.AttachUnitsToTransports( { cdrUnit }, { A_Air_Transport_1 } )

        end


        --[[  UNLOADING ATTACKER FROM AIR TRANSPORT  ]]--

        if not cdrUnit:IsDead() and cdrUnit:IsUnitState('Attached') then

            --[[  IssueTransportUnload( transports, landingLocation )                          ]]--
            --[[  transports is a set of transports, landingLocation = is a marker's position  ]]--

            IssueTransportUnload( { A_Air_Transport_1 }, destMarkerPos )

            --[[  ACUisReady UNLOCKING INPUT TRIGGER !           ]]--
            --[[  it's waiting for the Transport to be Unloaded  ]]--

            local waitForTransportUnload = true
            local armyIndex = cdrUnit:GetArmy()

            WaitForFullUnload = ForkThread ( function(self)

                while waitForTransportUnload do
                    waitForTransportUnload = UnitsStillAttached( { cdrUnit } , { A_Air_Transport_1 } )
                end

                if not waitForTransportUnload then
                    LOG("MANIMAL:  DEBUG ACUisReady = TRUE !!! .")
                    ubcsTable[armyIndex].isReady = true
                    ACUisReady = true
                end

                KillThread(self)

            end)

        end

    end

end




--[[                                                                             ]]--
--[[   UnitsStillAttached(units, transports)                                   ]]--
--[[                                                                             ]]--
--[[   inspired from ScenarioFramework.lua                                       ]]--
--[[   function PlatoonAttackWithTransportsThread                                ]]--
--[[                   ( platoon, landingChain, attackChain, instant )           ]]--
--[[                                                                             ]]--

function UnitsStillAttached(units, transports)

    local attached = true

    while attached do

        WaitSeconds(3)

        local allDead = true

        for k,v in transports do
            if not v:IsDead() then
                allDead = false
                break
            end
        end

        if allDead then
           return
        end

        attached = false

        for num, unit in units do
            if not unit:IsDead() and unit:IsUnitState('Attached') then
                attached = true
                break
            end
        end

    end

    return attached

end




--[[                                                                             ]]--
--[[  AirSpiesPatrouilleDropZone()                                               ]]--
--[[                                                                             ]]--
--[[  Patrouille AirSpies autour de la Drop Zone                                 ]]--
--[[                                                                             ]]--

function AirSpiesPatrouilleDropZone()

    --[[  Spawn Air Spies for Attacker ACU  ]]--

    local spawnPosLoc   = mPos( 'DZ_P0_1' )
    local origMarkerPos = ScenarioUtils.MarkerToPosition( 'DZ_P0_1' )
    local destMarkerPos = ScenarioUtils.MarkerToPosition( 'DZ_P1' )
    local attackChain   = 'Drop_Zone_1_Patrol'
			
    # Creer 10 SpyPlanes Cybran Tech 2
    local unitcount = 1
    local Air_Spy = false

    while  unitcount != 11  do

        Air_Spy = A_Air_Spies_1[unitcount]

        if not Air_Spy then

            Air_Spy = CreateUnitHPR( unitsTable[2].unitId, unitsTable[2].army, spawnPosLoc[1], spawnPosLoc[2], spawnPosLoc[3], 0, 0, 0 )

            if Air_Spy != nil then
                LOG("MANIMAL:  SPAWN � la DZ_P0 A_Air_Spies_1 n� " .. unitcount .. " .")
                A_Air_Spies_1[unitcount] = Air_Spy
                unitcount = unitcount + 1

            end

        end

    end

    # successful spawn so the Spies can go to Patrol !
    if A_Air_Spies_1 != nil then

        LOG("MANIMAL:  SPAWN de A_Air_Spies_1 .")

        --[[  Patrouille AirSpies autour de la Drop Zone  ]]--

        local attackLocs = ScenarioUtils.ChainToPositions(attackChain)

        for k,v in attackLocs do
            LOG("MANIMAL:  A_Air_Spies_1 en PATROUILLE sur Drop_Zone_1_Patrol, point de passage n�" .. k .. " .")
            IssuePatrol( A_Air_Spies_1, v )
        end

    end

end




--[[                                                                             ]]--
--[[  ArmerLanceMissilesTactique()                                               ]]--
--[[                                                                             ]]--
--[[  Creer et Armer le Silo Lance Missiles Tactique pour DEFENDER               ]]--
--[[                                                                             ]]--

function ArmerLanceMissilesTactique()

    --[[  Spawn Tactical Missiles Launcher for Defender ACU  ]]--

    local spawnPosLoc = mPos( 'TML_P0' )
			
    D_Tactical_Missiles_1 = CreateUnitHPR( unitsTable[3].unitId, unitsTable[3].army, spawnPosLoc[1], spawnPosLoc[2], spawnPosLoc[3], 0, 0, 0 )

    # successful spawn so the transport can be loaded up !
    if D_Tactical_Missiles_1 != nil then

        LOG("MANIMAL:  SPAWN de D_Tactical_Missiles_1 .")

        --[[  BUILDING TACTICAL MISSILES (FULL SILO)  ]]--

        BuildMissiles(D_Tactical_Missiles_1, 6)

    end

end




--[[                                                                             ]]--
--[[   BuildMissiles(unit, NumToBuild)                                           ]]--
--[[                                                                             ]]--
--[[   from PerfTest (altered for custom use)                                    ]]--
--[[   Attempts to build Missiles in an existent unit (i.e. SILO)                ]]--
--[[                                                                             ]]--

function BuildMissiles(unit, NumToBuild)

    if unit then
        LOG("MANIMAL:  Building Tactical Missiles with Number of Missiles = " .. NumToBuild .. " .")
        unit:GiveTacticalSiloAmmo(NumToBuild)
    else
        error('*ERROR* Attempting to build Missiles in a non-existent unit: '.. arg[arg['n']], 2)
    end

end




--[[                                                                             ]]--
--[[  StartSoloACU()                                                             ]]--
--[[                                                                             ]]--
--[[  Start the Custom Scenario                                                  ]]--
--[[                                                                             ]]--

function StartSoloACU()

end





--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[   UTILITY FUNCTIONS                                                         ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[ ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ ]]--



--[[                                                                             ]]--
--[[   GetMarker(name)                                                           ]]--
--[[                                                                             ]]--
--[[   get marker given a name from map_save done in map editor                  ]]--
--[[   you can the use marker.position to get a location in the {x,y,z} format   ]]--
--[[                                                                             ]]--

function GetMarker(name)
    return Scenario.MasterChain._MASTERCHAIN_.Markers[name]
end



--[[                                                                             ]]--
--[[   mPos(name)                                                                ]]--
--[[                                                                             ]]--
--[[   Get Marker position                                                       ]]--
--[[                                                                             ]]--

function mPos(name)
    return GetMarker(name).position
end



--[[                                                                             ]]--
--[[   strEnemi(iArmy)                                                           ]]--
--[[                                                                             ]]--
--[[   Returns the enemi of the current Army                                     ]]--
--[[                                                                             ]]--

function strEnemi(iArmy)
    return ubcsTable[iArmy].enemi
end




--[[                                                                             ]]--
--[[   CameraTrackEntity( unit, seconds, unlock )                                ]]--
--[[                                                                             ]]--
--[[   d�rivable en CameraTrackEntities( { unit1, unit2, ... }, unlock )         ]]--
--[[                                                                             ]]--

function CameraTrackEntity( unit, seconds, unlock )
    ForkThread( CamTrackEntity, unit, seconds, unit:GetHeading(), unlock )
end




--[[                                                                             ]]--
--[[   CamTrackEntity( unit, seconds, heading, unlock )                          ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--
--[[                                                                             ]]--

function CamTrackEntity( unit, seconds, heading, unlock )
    local cam = import('/lua/simcamera.lua').SimCamera('WorldCamera')
    LockInput()
    cam:UseSystemClock()
    WaitTicks(1)
    # Track the unit
    if unit then
        local zoomVar = 50
        if EntityCategoryContains( categories.uaa0310, unit ) then
            zoomVar = 150
        end
        local pos = unit:GetPosition()
        local marker = {
            orientation = VECTOR3( heading, .5, 0 ),
            position = { pos[1], pos[2] - 15, pos[3] },
            zoom = zoomVar,
        }

        # function CameraTrackEntities( units, zoom, seconds )
        # This will make the camera track a group of entities.
        # The "seconds" field is how long it will take to get in place.
        # After that the camera will follow that unit until told to do something else.

        cam:TrackEntity( unit, zoomVar, seconds)
    end
    if (unlock) then
        WaitSeconds(seconds)
        #cam:Reset()  # <<====== SEMBLE INEFFICACE !!!
        UnlockInput()
    end
end




